@@ -1,4 +1,4 @@
-/* Copyright (c) 2007-2011, The Tor Project, Inc. */
+/* Copyright (c) 2007-2012, The Tor Project, Inc. */
 /* See LICENSE for licensing information */
 
 #include "orconfig.h"
@@ -9,16 +9,11 @@ typedef unsigned int uint;
 #ifndef HAVE_U_CHAR
 typedef unsigned char u_char;
 #endif
-#ifdef MS_WINDOWS
+#ifdef _WIN32
 #define inline __inline
 #endif
 #include "torint.h"
 
-#if defined(MS_WINDOWS) && !defined(WIN32)
-/* How did _this_ happen? */
-#define WIN32
-#endif
-
 /* These are for debugging possible memory leaks. */
 #include "util.h"
 #include "compat.h"
